# -*-  mode: shell-script; sh-shell: sh;  -*-
# @configure-input@

# run-example.as -- run an example file specifed by $1, outputting to
# $2

######################################################################
# Mu (Miscellaneous Utilities) is a general convenience library.     #
# Copyright (C) 2020  Asher Gordon <AsDaGo@posteo.net>               #
#                                                                    #
# This file is part of Mu.                                           #
#                                                                    #
# Mu is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by  #
# the Free Software Foundation, either version 3 of the License, or  #
# (at your option) any later version.                                #
#                                                                    #
# Mu is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY #
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public   #
# License for more details.                                          #
#                                                                    #
# You should have received a copy of the GNU General Public License  #
# along with Mu.  If not, see <https://www.gnu.org/licenses/>.       #
######################################################################

AS_INIT

# Allow environment variables to take precedence.
: ${SED="@SED@"}
: ${GREP="@GREP@"}
: ${RM="rm -f"}
: ${MV="mv -f"}

# Act like 'grep -q' since not all greps support '-q'.
fn_grep_q() {
    $GREP "$@" > /dev/null 2>&1
}

# This command escapes Texinfo metacharacters.
escape="$SED -e s,@,@@,g
	     -e s,[{}],@&,g
	     -e s,/\*\(@.*\)\*/,\1,g
	     -e s/\(@[a-z][a-z]*\)@{\([^}]*\)@}/\1{\2}/g
"

# This command corrects the argv[0] output as a result of Libtools
# wrapper scripts.
correct_argv0="$SED s,/.*\.libs/\(..*\),./\1,"

AS_IF([test $# -ne 2],
     [AS_ECHO(["Usage: $0 INPUT OUTPUT"])
      AS_EXIT([1])])

# Remove the output file first.
$RM $2

# Print a generation notice.
AS_ECHO(['@c DO NOT EDIT!']) >> $2
AS_ECHO(["@c $2 generated by $0 from $1."]) >> $2

save_IFS=$IFS
IFS='
'
for line in `cat $1`; do
    IFS=$save_IFS

    # Translate shell comments to Texinfo comments. Three pairs of
    # brackets for [[[:space:]]] because the outer pair quotes it for
    # M4.
    AS_IF([AS_ECHO(["$line"]) | fn_grep_q '^[[[:space:]]]*#'],
         [AS_ECHO(["$line"]) | $SED 's/^/@c /' >> $2
	  continue])

    # Echo the command to the output file, and then run it, indicating
    # standard output with @print{} and standard error with @error{}.
    AS_ECHO(["\$ $line"]) | $escape >> $2
    eval "$line" > $2.out 2> $2.err
    status=$?

    # The $SED commands replace empty lines with a single space. This
    # is so that we will see empty lines when we iterate over them
    # with IFS set to a single newline.
    out=`cat $2.out | $SED 's/^$/ /g'`
    err=`cat $2.err | $SED 's/^$/ /g'`
    $RM $2.out $2.err

    # Exit if any command wasn't found (127 means command not
    # found). Also try and detect when the Libtool wrappers couldn't
    # find the real executable.
    AS_IF([test $status -eq 127 ||					\
	       { test $status -ne 0 &&					\
		     AS_ECHO(["$err"]) |				\
			 fn_grep_q '^This script is just a wrapper for '; }],
	 [AS_ECHO(["$err"]) 1>&2
	  AS_EXIT([$status])])

    # Print the output first and then the error.
    save_IFS=$IFS
    IFS='
'
    for out_line in $out; do
        IFS=$save_IFS
        AS_ECHO(["$out_line"]) | $correct_argv0 | \
            $escape | sed 's/^/@print{} /' >> $2
    done
    IFS='
'
    for err_line in $err; do
        IFS=$save_IFS
        AS_ECHO(["$err_line"]) | $correct_argv0 | \
            $escape | sed 's/^/@error{} /' >> $2
    done
    IFS=$save_IFS
done
IFS=$save_IFS
